﻿using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Text.Encodings.Web;
using System.Text.Json;
using System.Text.Unicode;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Threading;
using Txgy.FilesWatcher.model;

namespace Txgy.FilesWatcher.ViewModels
{
    public class ManualModeDialogViewModel : BindableBase, IDialogAware
    {
        public ManualModeDialogViewModel(IEventAggregator ea)
        {
            _log = new GetLogger("ManualModeDialogViewModel");
            _log.Log.Debug("ManualModeDialogViewModel() ...");
        }
        private GetLogger _log;
        public string Title => "手动模式";

        public event Action<IDialogResult> RequestClose;

        public bool CanCloseDialog() => true;

        public void OnDialogClosed()
        {
            cts.Cancel();
        }
        public MainViewModel mainModel;
        public void OnDialogOpened(IDialogParameters parameters)
        {
            StartTime = DateTime.Now.AddDays(-3);
            EndTime = StartTime.AddDays(4);
            mainModel = parameters.GetValue<MainViewModel>("main");
            if (DataList.Count > 0) { DataList.Clear(); }
            if (RealTimeDataList.Count >= 0) { RealTimeDataList.Clear(); }
            if (PostDataList.Count >= 0) { PostDataList.Clear(); }
        }

        public ICommand SelectFileCommand { get; set; }
        public ICommand UploadCommand { get; set; }


        private ObservableCollection<FileModel> dataList = new ObservableCollection<FileModel>();

        public ObservableCollection<FileModel> DataList
        {
            get { return dataList; }
            set { SetProperty(ref dataList, value); }
        }
        private ObservableCollection<FileModel> realTimeDataList = new ObservableCollection<FileModel>();

        public ObservableCollection<FileModel> RealTimeDataList
        {
            get { return realTimeDataList; }
            set { SetProperty(ref realTimeDataList, value); }
        }

        private ObservableCollection<FileModel> postDataList = new ObservableCollection<FileModel>();

        public ObservableCollection<FileModel> PostDataList

        {
            get { return postDataList; }
            set { SetProperty(ref postDataList, value); }
        }

        private DateTime startTime;
        public DateTime StartTime { get => startTime; set => SetProperty(ref startTime, value); }
        private DateTime endTime;
        public DateTime EndTime { get => endTime; set => SetProperty(ref endTime, value); }
        private string stateMes="未开始";
        public string StateMes { get => stateMes; set => SetProperty(ref stateMes, value); }
        private bool startEnable = true;
        public bool StartEnable { get => startEnable; set => SetProperty(ref startEnable, value); }

        public DelegateCommand StartCommand => new DelegateCommand(async () =>
        {
            //if ((EndTime - StartTime).TotalDays > 4)
            //{
            //    MessageBox.Show("时间区间超范围!");
            //}
            StateMes = "执行中";
            StartEnable = false;
            await StartHandle(StartTime,EndTime);
            StateMes = "未开始";
            StartEnable = true;
        }).ObservesCanExecute(()=>StartEnable);

        CancellationTokenSource cts = new CancellationTokenSource();
        public async Task StartHandle(DateTime start, DateTime end)
        {
            _log.Log.DebugFormat("ManualModeDialogViewModel.StartHandle() start:{0},end:{1}",start, end);
            CancellationToken cancellationToken = cts.Token;
            string realtimePath = Path.Combine(mainModel.MainPath, mainModel.RealtimePath);
            string postPath = Path.Combine(mainModel.MainPath, mainModel.PostPath);
            Task task1 = Task.Run(() =>
            {
                DateTime time = start.AddMinutes(1);
                while (!cancellationToken.IsCancellationRequested)
                {
                    time = time.AddMinutes(1);
                    MseedUpload(time);
                    if (time >= end)
                    {
                        break;
                    }
                }
            }, cancellationToken);
            Task task2 = Task.Run(() =>
            {
                var files = new DirectoryInfo(realtimePath).GetFiles("*.index",SearchOption.AllDirectories);
                foreach (var file in files)
                {
                    var lines = File.ReadAllLines(file.FullName);
                    foreach (var line in lines)
                    {
                        if (cancellationToken.IsCancellationRequested)
                        {
                            Console.WriteLine("Cancellation");
                            break;
                        }
                        RealtimeChanged(file.FullName, line);
                    }
                }
            }, cancellationToken);
            Task task3 = Task.Run(() =>
            {
                var postFiles = new DirectoryInfo(postPath).GetFiles("*.index", SearchOption.AllDirectories);
                foreach (var file in postFiles)
                {
                    var lines = File.ReadAllLines(file.FullName);
                    foreach (var line in lines)
                    {
                        if (cancellationToken.IsCancellationRequested)
                        {
                            Console.WriteLine("Cancellation");
                            break;
                        }
                        PostChanged(file.FullName, line);
                    }
                }
            }, cancellationToken);
            await Task.WhenAll(task1, task2, task3);
        }


        public DelegateCommand ManualCommand => new DelegateCommand(() =>
        {
            cts.Cancel();
        });

        private void MseedUpload(DateTime currentT)
        {
            string path = $"{Path.Combine(mainModel.MainPath, mainModel.MseedPath)}/{currentT.Month.ToString("D2")}/{currentT.Day.ToString("D2")}/{currentT.Hour.ToString("D2")}/{currentT.Minute.ToString("D2")}";
            // path = "I:\\yuwuN3102mseed\\NET2023\\06\\22\\21\\51";
            Debug.WriteLine($"**********当前时间：{currentT},监控路径:{path}");
            if (Directory.Exists(path) && mainModel.IsUploadDB && mainModel.IsUploadMseedPath)
            {
                _log.Log.DebugFormat("ManualModeDialogViewModel.MseedUpload() currentT:{0},path:{1}", currentT, path);
                UploadMseedFile.UploadMSeedOnce((a =>
                {
                    timer1.Dispatcher.Invoke(() =>
                    {
                       // if (DataList.Count >= 20) { DataList.Remove(dataList.Last()); }
                        DataList.Insert(0, new FileModel
                        {
                            CreateTime = DateTime.Now.ToString("yyyy-MM-dd T HH:mm:ss"),
                            Data = $"{currentT.AddMinutes(-2).ToString("yyyy-MM-dd T HH:mm:ss")}{a}"
                        });
                    });
                }), path, mainModel.WorkAreaId);
            }
        }

        private void RealtimeChanged(string filePath, string lastLine)
        {
            // string tmp = dt.Hour.ToString() + "时" + dt.Minute.ToString() + "分" + dt.Second.ToString() + "秒" + dt.Millisecond.ToString() + "毫秒,目录发生变化\r\n";         
            DateTime lastWriteTime = DateTime.Now;  //File.GetLastWriteTime(e.FullPath);
            Debug.WriteLine($"最后修改时间：{lastWriteTime},文件路径：{filePath}");
            filePath = Path.GetDirectoryName(filePath);
            _log.Log.DebugFormat("ManualModeDialogViewModel.RealtimeChanged() filePath:{0},path:{1}", filePath, lastLine);
            if (mainModel.IsUploadDB && mainModel.IsUploadRealtimePath)
            {
                UploadRealtimeFile.UploadRealtimeFileOnce(filePath, lastLine, mainModel.WorkAreaId, out bool isUpload);
                timer1.Dispatcher.Invoke(() =>
                {
                    // if (RealTimeDataList.Count >= 20) { RealTimeDataList.Remove(RealTimeDataList.Last()); }
                    if (isUpload)
                    {
                        RealTimeDataList.Insert(0, new FileModel
                        {
                            CreateTime = DateTime.Now.ToString("yyyy-MM-dd T HH:mm:ss"),
                            Data = $"{lastLine},事件上传成功。"
                        });
                    }
                });
            }
            if (mainModel.IsUploadMQTT && mainModel.IsUploadRealtimePath)
            {
                UploadRealtimeFile.UploadRealtimeFileMQ((a, b) =>
                {

                }, filePath, lastLine);
            }
        }


        private void PostChanged(string filePath, string lastLine)
        {
            // string tmp = dt.Hour.ToString() + "时" + dt.Minute.ToString() + "分" + dt.Second.ToString() + "秒" + dt.Millisecond.ToString() + "毫秒,目录发生变化\r\n";         
            DateTime lastWriteTime = DateTime.Now;  //File.GetLastWriteTime(e.FullPath);
            Debug.WriteLine($"最后修改时间：{lastWriteTime},文件路径：{filePath}");
            filePath = Path.GetDirectoryName(filePath);
           _log.Log.DebugFormat("ManualModeDialogViewModel.RealtimeChanged() filePath:{0},path:{1}", filePath, lastLine);
            if (mainModel.IsUploadDB && mainModel.IsUploadPostPath)
            {
                UploadPostproFile.UploadPostproFileOnce(filePath, lastLine, mainModel.WorkAreaId, out bool isUpload);
                {
                    timer1.Dispatcher.Invoke(() =>
                    {
                        //  if (PostDataList.Count >= 20) { PostDataList.Remove(PostDataList.Last()); }
                        if (isUpload)
                        {
                            PostDataList.Insert(0, new FileModel
                            {
                                CreateTime = DateTime.Now.ToString("yyyy-MM-dd T HH:mm:ss"),
                                Data = $"{lastLine},事件上传成功。"
                            });
                        }
                    });
                }
            }
            if (mainModel.IsUploadMQTT && mainModel.IsUploadPostPath)
            {
                UploadPostproFile.UploadPostproFileMQ((a, b) =>
                {
                    timer1.Dispatcher.Invoke(() =>
                    {

                    });
                }, filePath, lastLine);
            }
        }

        private DispatcherTimer timer1 = new DispatcherTimer();
        private System.Timers.Timer timerTool = new System.Timers.Timer();
        private readonly SystemConfig _systemConfig;
        private readonly IEventAggregator _ea;
        private readonly IDialogService _dialogService;
        #region
        private void TimerTool_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            DateTime currentT = DateTime.Now;
            string path = $"{mainModel.ToolPath}/{currentT.Month.ToString("D2")}/{currentT.Day.ToString("D2")}/{currentT.Hour.ToString("D2")}/{currentT.AddMinutes(-2).Minute.ToString("D2")}";
            // path = "H:/mzhifa/txgy/FileWatchProject/NET2023/11/29/00/06";
            Debug.WriteLine($"**********apmTools：{currentT},结果路径:{path}");
            if (Directory.Exists(path) && mainModel.IsEnableToolPath)
            {
                CMDStartProcess(_systemConfig.proTools, path);
            }
        }

        private bool CMDStartProcess(ProcessInfo proInfo, string path)
        {
            Process process = new Process();
            process.Exited += Process_Exited;
            process.EnableRaisingEvents = true;
            process.StartInfo.FileName = Path.GetFullPath(proInfo.ProPath + proInfo.ProName + ".exe");
            process.StartInfo.UseShellExecute = false;
            process.StartInfo.WorkingDirectory = Path.GetFullPath(proInfo.ProPath);
            process.StartInfo.Arguments = proInfo.ProParams + path;
            process.StartInfo.RedirectStandardOutput = true;
            process.StartInfo.RedirectStandardError = true;

            process.StartInfo.CreateNoWindow = true;
            //* Set your output and error (asynchronous) handlers
            process.OutputDataReceived += new DataReceivedEventHandler(OutputHandler);
            process.ErrorDataReceived += new DataReceivedEventHandler(OutputHandler);

            bool res = process.Start();

            return res;
        }

        private void Process_Exited(object? sender, EventArgs e)
        {

        }
        void OutputHandler(object sendingProcess, DataReceivedEventArgs outLine)
        {
            Debug.WriteLine("output*************:{0},{1}", sendingProcess.ToString(), outLine.Data);
            if (string.IsNullOrEmpty(outLine.Data)) return;
        }
        #endregion


    }
}
