﻿using Dapper;
using DryIoc;
using Google.Protobuf.WellKnownTypes;
using HandyControl.Controls;
using Microsoft.Xaml.Behaviors.Media;
using MySql.Data.MySqlClient;
using Org.BouncyCastle.Bcpg.OpenPgp;
using Org.BouncyCastle.Pqc.Crypto.Lms;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Encodings.Web;
using System.Text.Json;
using System.Text.Unicode;
using System.Threading.Tasks;
using System.Windows.Markup;
using System.Xml.Linq;
using Txgy.FilesWatcher.DBModels;

namespace Txgy.FilesWatcher.model
{
    public class UploadRealtimeFile
    {
        public static void UploadRealtimeFileOnce(string path, string eventMessage, int workAreaId, out bool isUpload)
        {
            GetLogger _log = new GetLogger("UploadRealtimeFile");
            _log.Log.DebugFormat("UploadRealtimeFile.UploadRealtimeFileOnce() path:{0},eventMessage:{1},workAreaId:{2}", path, eventMessage, workAreaId);
            isUpload = false;
            try
            {
                var currentTime = DateTime.Now;
                var mseedFile = new DirectoryInfo(path).GetFiles("*.mseed");
                var Dfiles = mseedFile.Where(f => eventMessage.Contains($"{f.Name.Substring(3, 4)}-{f.Name.Substring(7, 2)}-{f.Name.Substring(9, 2)}T{f.Name.Substring(12, 2)}:{f.Name.Substring(14, 2)}:{f.Name.Substring(16, 2)}"));
                var files = Directory.GetFiles(path);
                foreach (var dFile in Dfiles)
                {
                    string file = dFile.Name;
                    string tbrealtimeresult = "realtimeeventresult";
                    string tbname = "realtimewavedatas";
                    string uploadedtbname = "uploadedrealtime";
                    string sqlNumber = $"SELECT COUNT(*) FROM {uploadedtbname} WHERE filename = '{file}'";
                    //检查文件是否已经上传
                    using (var conn = DataBaseConnect.GetInstance.OpenConnection())
                    {
                        MySqlCommand cmd = new MySqlCommand(sqlNumber, conn);
                        var a = conn.QueryFirst<int>(sqlNumber);
                        if (a == 0)
                        {
                            string[] index_file_line = eventMessage.Split(" ", StringSplitOptions.RemoveEmptyEntries);
                            if (index_file_line[4] == "None")
                            {
                                index_file_line[5] = "-4";
                            }

                            string EventTime = index_file_line[0].Substring(0, 23);
                            string OriginTime = EventTime;
                            //.json文件
                            string JPath = dFile.FullName.Replace(".mseed", "A.json");
                            if(File.Exists(JPath))
                            {
                                string mes= File.ReadAllText(JPath);
                                if (!mes.Contains(EventTime))
                                {
                                    //和json文件的otime 时间需要一致
                                    continue;
                                }
                            }

                            // 上传前，检查workAreaId和eventTime                           
                            sqlNumber = $"SELECT COUNT(*) FROM {tbrealtimeresult} WHERE WorkAreaId = '{workAreaId}' && EventTime='{EventTime}'";
                            cmd = new MySqlCommand(sqlNumber, conn);
                            int b = conn.QueryFirst<int>(sqlNumber);
                            if (b == 0)
                            {
                                sqlNumber = $"INSERT INTO {tbrealtimeresult}(WorkAreaId, EventTime, OriginTime, X, Y, Z, ML, LocSta, MLSta, RMS) VALUES('{workAreaId}', '{EventTime}', '{OriginTime}', '{index_file_line[1]}', '{index_file_line[2]}', '{index_file_line[3]}', '{index_file_line[5]}', '{index_file_line[7]}', '{index_file_line[8]}', '{index_file_line[6]}')";
                                conn.Execute(sqlNumber);
                                _log.Log.DebugFormat("UploadRealtimeFileOnce() 上传数据表:{0},eventMessage:{1}", tbrealtimeresult, eventMessage);
                            }                        
                            isUpload = true;
                            // .mseed文件
                            FileStream fs = new FileStream(dFile.FullName, FileMode.Open, FileAccess.Read);
                            BinaryReader mbr = new BinaryReader(fs);
                            Byte[] mseedDatas = mbr.ReadBytes((int)fs.Length);
                            fs.Close();
                            //.json文件
                            string JsonPath = dFile.FullName.Replace(".mseed", "A.json");
                            fs = new FileStream(JsonPath, FileMode.Open, FileAccess.Read);
                            BinaryReader jbr = new BinaryReader(fs);
                            Byte[] jsonDatas = jbr.ReadBytes((int)fs.Length);
                            string jsonString = System.Text.Encoding.Default.GetString(jsonDatas);
                            fs.Close();
                            var options = new JsonSerializerOptions
                            {
                                // 整齐打印
                                WriteIndented = true,
                                //重新编码，解决中文乱码问题
                                Encoder = JavaScriptEncoder.Create(UnicodeRanges.All)
                            };
                            sqlNumber = $"SELECT COUNT(*) FROM {tbname} WHERE EventTime = '{EventTime}'";
                            MySqlCommand md = new MySqlCommand(sqlNumber, conn);
                            var count = conn.QueryFirst<int>(sqlNumber);
                            if (count == 0)
                            {
                                sqlNumber = $"INSERT INTO {tbname} (EventTime, WorkAreaID, WaveData, JsonFile) VALUES('{EventTime}', '{workAreaId}', @mDatas, @jDatas);";
                                MySqlCommand mycomm = new MySqlCommand(sqlNumber, conn);
                                conn.Execute(sqlNumber, new { mDatas = mseedDatas, jDatas = jsonDatas });
                                _log.Log.DebugFormat("UploadRealtimeFileOnce() 上传数据表:{0}", tbname);
                            }
                            sqlNumber = $"INSERT INTO {uploadedtbname}(filename, uploadtime) VALUES('{file}', '{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}')";
                            int res = conn.Execute(sqlNumber);
                            _log.Log.DebugFormat("UploadRealtimeFileOnce() 上传数据表:{0},文传:{1}", uploadedtbname, file);

                        }
                    }
                }
            }
            catch (Exception ex)
            {
                _log.Log.ErrorFormat("UploadRealtimeFile.UploadRealtimeFileOnce() error:{0}", ex);
            }
        }

        public static void UploadRealtimeFileMQ(Action<string, string> mqAction, string path, string eventMessage)
        {
            GetLogger _log = new GetLogger("UploadRealtimeFile");
            _log.Log.DebugFormat("UploadRealtimeFile.UploadRealtimeFileMQ() path:{0},eventMessage:{1}", path, eventMessage);
            try
            {
                var currentTime = DateTime.Now;
                var mseedFile = new DirectoryInfo(path).GetFiles("*.mseed");
                var Dfiles = mseedFile.Where(f => eventMessage.Contains($"{f.Name.Substring(3, 4)}-{f.Name.Substring(7, 2)}-{f.Name.Substring(9, 2)}T{f.Name.Substring(12, 2)}:{f.Name.Substring(14, 2)}:{f.Name.Substring(16, 2)}"));
                var files = Directory.GetFiles(path);
                foreach (var dFile in Dfiles)
                {
                    string file = dFile.Name;

                    //.json文件
                    string JsonPath = dFile.FullName.Replace(".mseed", "A.json");
                    FileStream fs = new FileStream(JsonPath, FileMode.Open, FileAccess.Read);
                    BinaryReader jbr = new BinaryReader(fs);
                    Byte[] jsonDatas = jbr.ReadBytes((int)fs.Length);
                    string jsonString = Encoding.Default.GetString(jsonDatas);
                    fs.Close();
                    var options = new JsonSerializerOptions
                    {
                        // 整齐打印
                        WriteIndented = true,
                        //重新编码，解决中文乱码问题
                        Encoder = JavaScriptEncoder.Create(UnicodeRanges.All)
                    };
                    mqAction($"/watcherdata/realtime", JsonSerializer.Serialize(new { jsonFile = jsonString, eventMessage = eventMessage }, options));
                }
            }
            catch (Exception ex)
            {
                _log.Log.ErrorFormat("UploadRealtimeFile.UploadRealtimeFileMQ() error:{0}", ex);
            }
        }

    }
}
